import os.path
import shutil
from dataclasses import dataclass
from pathlib import Path

from workstation import RESOURCES_PATH
from workstation.os_manager import OSManager
from workstation.zsh import ZSH_CONFIG_PATH

SDKMAN_ROOT = os.path.expanduser("~/.sdkman")
SDKMAN_ZSH_CONFIG_LOCATION = os.path.join(ZSH_CONFIG_PATH, "sdkman.zsh")
SDKMAN_ZSH_CONFIG_RESOURCE_LOCATION = os.path.join(
    RESOURCES_PATH, "zshrc", "sdkman.zsh"
)


@dataclass
class SDKManInstaller:
    os_manager: OSManager

    @staticmethod
    def _remove_sdkman():
        shutil.rmtree(SDKMAN_ROOT, ignore_errors=True)

    def _install_sdkman(self):
        self._remove_sdkman()
        cmd = f'export SDKMAN_DIR="{SDKMAN_ROOT}" && curl -s "https://get.sdkman.io?rcupdate=false" | bash'
        self.os_manager.run(cmd)

    def _install_java(self, java_version: str):
        cmds = [
            "sdk install maven",
            f"sdk install java {java_version}",
            f"sdk default java {java_version}",
        ]
        for cmd in cmds:
            self.os_manager.run_in_zsh(cmd)

    @staticmethod
    def _remove_sdkman_zsh_config():
        Path(SDKMAN_ZSH_CONFIG_LOCATION).unlink(missing_ok=True)

    def _setup_sdkman_zsh_config(self):
        self._remove_sdkman_zsh_config()
        shutil.copy(SDKMAN_ZSH_CONFIG_RESOURCE_LOCATION, SDKMAN_ZSH_CONFIG_LOCATION)

    def run(self, java_version: str):
        self._install_sdkman()
        self._setup_sdkman_zsh_config()
        self._install_java(java_version=java_version)



from dataclasses import dataclass

import requests

from workstation import OS_FEDORA
from workstation.os_manager import Fedora
from workstation.os_manager import OSManager


@dataclass
class MiscellaneousInstaller:
    os_manager: OSManager

    def _install_jetbrains_toolbox(self):
        if self.os_manager.operating_system == OS_FEDORA:
            os_manager: Fedora = self.os_manager
            r = requests.get(
                "https://data.services.jetbrains.com//products/releases?code=TBA&latest=true&type=release"
            )
            url = r.json()["TBA"][0]["downloads"]["linux"]["link"]
            os_manager.tgz_install_from_url(url, "jetbrains-toolbox")

    def run(self):
        self._install_jetbrains_toolbox()



import click
import distro

from workstation.nvm import NVMInstaller
from workstation.os_manager import Fedora
from workstation.python import PythonInstaller
from workstation.sdkman import SDKManInstaller
from workstation.terminal import TilixInstaller
from workstation.zsh import ZSHInstaller


@click.command()
def install_workstation():
    if distro.id() == "fedora":
        os_manager = Fedora()
    else:
        raise ValueError("OS not supported yet!")

    if click.confirm("Do you want to setup Zsh?"):
        zsh_installer = ZSHInstaller(os_manager=os_manager)
        zsh_installer.run()

    if click.confirm("Do you want to install Tilix?"):
        zsh_installer = TilixInstaller(os_manager=os_manager)
        zsh_installer.run()

    if click.confirm("Do you want to setup Python?"):
        python_installer = PythonInstaller(os_manager=os_manager)
        choice = click.prompt(
            "Please select python versions:",
            type=click.Choice(["3.8.12", "3.9.7"]),
            show_default=False,
        )
        python_installer.run(python_versions=[choice])

    if click.confirm("Do you want to setup Node?"):
        nvm_installer = NVMInstaller(os_manager=os_manager)
        choice = click.prompt(
            "Please select node versions:",
            type=click.Choice(["v16.13.0"]),
            show_default=False,
        )
        nvm_installer.run(node_version=choice)

    if click.confirm("Do you want to setup Java?"):
        sdkman_installer = SDKManInstaller(os_manager=os_manager)
        choice = click.prompt(
            "Please select java versions:",
            type=click.Choice(["14.0.2-open"]),
            show_default=False,
        )
        sdkman_installer.run(java_version=choice)



import os
import shutil
from dataclasses import dataclass
from pathlib import Path

from loguru import logger

from workstation import RESOURCES_PATH
from workstation.os_manager import OSManager

ZSHRC_PATH = os.path.expanduser("~/.zshrc")
ZSH_CONFIG_PATH = os.path.expanduser("~/.config/zsh/config.d")
OH_MY_ZSH_PATH = os.path.expanduser("~/.oh-my-zsh")


@dataclass
class ZSHInstaller:
    os_manager: OSManager

    def _install_zsh(self):
        logger.info("Installing zsh")
        self.os_manager.native_install("zsh fd-find", sudo=True)

    def _install_oh_my_zsh(self):
        logger.info("Installing oh-my-zsh")
        shutil.rmtree(OH_MY_ZSH_PATH, ignore_errors=True)
        cmd = (
            'sh -c "$(curl -fsSL https://raw.githubusercontent.com/ohmyzsh/ohmyzsh/master/tools/install.sh)" "" '
            "--unattended "
        )
        self.os_manager.run(cmd)

    def _install_oh_my_zsh_plugins(self):
        logger.info("Installing zsh plugins")
        cmds = [
            "git clone https://github.com/zsh-users/zsh-syntax-highlighting.git ${"
            "ZSH_CUSTOM:-~/.oh-my-zsh/custom}/plugins/zsh-syntax-highlighting",
            "git clone https://github.com/zsh-users/zsh-autosuggestions ${"
            "ZSH_CUSTOM:-~/.oh-my-zsh/custom}/plugins/zsh-autosuggestions",
            "git clone https://github.com/zsh-users/zsh-completions ${"
            "ZSH_CUSTOM:=~/.oh-my-zsh/custom}/plugins/zsh-completions ",
        ]
        for cmd in cmds:
            self.os_manager.run(cmd)

    @staticmethod
    def _setup_zsh_config():
        logger.info("Setup zsh")
        Path(ZSHRC_PATH).unlink(missing_ok=True)
        zshrc_location = os.path.join(RESOURCES_PATH, ".zshrc")
        shutil.copy(zshrc_location, ZSHRC_PATH)
        for conf in ["banner.zsh", "init.zsh"]:
            os.makedirs(ZSH_CONFIG_PATH, exist_ok=True)
            resource_path = os.path.join(RESOURCES_PATH, "zshrc", conf)
            local_path = os.path.join(ZSH_CONFIG_PATH, conf)
            Path(local_path).unlink(missing_ok=True)
            shutil.copy(resource_path, local_path)

    def _install_theme(self):
        logger.info("Setup zsh theme")
        cmd = (
            f"git clone --depth=1 https://github.com/romkatv/powerlevel10k.git {OH_MY_ZSH_PATH}/custom/themes"
            f"/powerlevel10k"
        )
        self.os_manager.run(cmd)
        p10k_location = os.path.join(RESOURCES_PATH, ".p10k.zsh")
        shutil.copy(p10k_location, os.path.expanduser("~/.p10k.zsh"))

    def _change_shell(self):
        logger.info("Changing default shell to zsh")
        self.os_manager.run("chsh -s /usr/bin/zsh")

    def run(self):
        try:
            self._install_zsh()
            self._install_oh_my_zsh()
            self._install_oh_my_zsh_plugins()
            self._install_theme()
            self._setup_zsh_config()
            self._change_shell()
        except KeyboardInterrupt:
            logger.info("\n\n\033[0;31m❌ Aborting setup!\033[0m")



# import importlib.resources as pkg_resources
# import os
# import shutil
# from dataclasses import dataclass
# from pathlib import Path
#
# from loguru import logger
#
# from workstation.utils import full_path, OSManager
#
# fishRC_PATH = os.path.expanduser("~/.fishrc")
# fish_CONFIG_PATH = os.path.expanduser("~/.configs")
# OH_MY_FISH_PATH = os.path.expanduser("~/.oh-my-fish")
#
# RESOURCES_PATH = str(pkg_resources.path("workstation", "resources"))
#
#
# @dataclass
# class FishInstaller:
#     os_manager: OSManager
#
#     def install_fish(self):
#         logger.info("Installing fish")
#         self.os_manager.native_install("fish", sudo=True)
#
#     def install_oh_my_fish(self):
#         logger.info("Installing oh-my-fish")
#         shutil.rmtree(OH_MY_fish_PATH, ignore_errors=True)
#         cmd = 'sh -c "$(curl -fsSL https://raw.githubusercontent.com/ohmyfish/ohmyfish/master/tools/install.sh)" "" ' \
#               '--unattended '
#         self.os_manager.run(cmd)
#
#     def install_fish_plugins(self):
#         logger.info("Installing fish plugins")
#         cmds = [
#             "git clone https://github.com/fish-users/fish-syntax-highlighting.git ${"
#             "fish_CUSTOM:-~/.oh-my-fish/custom}/plugins/fish-syntax-highlighting",
#             "git clone https://github.com/fish-users/fish-autosuggestions ${"
#             "fish_CUSTOM:-~/.oh-my-fish/custom}/plugins/fish-autosuggestions",
#             "git clone https://github.com/fish-users/fish-completions ${"
#             "fish_CUSTOM:=~/.oh-my-fish/custom}/plugins/fish-completions "
#         ]
#         for cmd in cmds:
#             self.os_manager.run(cmd)
#
#     @staticmethod
#     def setup_fish_config():
#         logger.info("Setup fish")
#         Path(fishRC_PATH).unlink(missing_ok=True)
#         fishrc_location = os.path.join(RESOURCES_PATH, ".fishrc")
#         shutil.copy(fishrc_location, fishRC_PATH)
#         shutil.rmtree(fish_CONFIG_PATH, ignore_errors=True)
#         fish_config_location = os.path.join(RESOURCES_PATH, "fishrc")
#         shutil.copytree(fish_config_location, os.path.join(fish_CONFIG_PATH, "fishrc"), dirs_exist_ok=True)
#
#     def install_theme(self):
#         logger.info("Setup fish theme")
#         cmd = f"git clone --depth=1 https://github.com/romkatv/powerlevel10k.git {OH_MY_fish_PATH}/custom/themes" \
#               f"/powerlevel10k"
#         self.os_manager.run(cmd)
#         p10k_location = os.path.join(RESOURCES_PATH, ".p10k.fish")
#         shutil.copy(p10k_location, os.path.expanduser("~/.p10k.fish"))
#
#     def change_shell(self):
#         logger.info("Changing default shell to fish")
#         self.os_manager.run("chsh -s /usr/bin/fish")
#
#     def run_full_setup(self):
#         try:
#             self.install_fish()
#             self.install_fish_plugins()
#             self.install_oh_my_fish()
#             self.install_theme()
#             self.setup_fish_config()
#         except KeyboardInterrupt:
#             logger.info("\n\n\033[0;31m❌ Aborting setup!\033[0m")
#
#



import os.path
import shutil
from dataclasses import dataclass

from workstation import RESOURCES_PATH
from workstation.os_manager import full_path
from workstation.os_manager import OSManager

DCONF_LOCATION = os.path.join(RESOURCES_PATH, "tilix.dconf")


@dataclass
class TilixInstaller:
    os_manager: OSManager

    def _install_tilix(self):
        self.os_manager.native_install("tilix", sudo=True)

    def _install_custom_theme(self):
        self.os_manager.run(f"dconf load /com/gexperts/Tilix/ <{DCONF_LOCATION}")
        shutil.rmtree(
            os.path.expanduser("~/.local/share/fonts-tmp"), ignore_errors=True
        )
        os.system(
            f"git clone https://github.com/romkatv/powerlevel10k-media.git {full_path('.local/share/fonts-tmp/')}"
        )
        os.makedirs(os.path.expanduser("~/.local/share/fonts"), exist_ok=True)
        files = os.listdir(os.path.expanduser("~/.local/share/fonts-tmp/"))
        for x in files:
            if x.startswith("Meslo"):
                shutil.copyfile(
                    os.path.join(os.path.expanduser("~/.local/share/fonts-tmp/"), x),
                    os.path.join(os.path.expanduser("~/.local/share/fonts"), x),
                )
        shutil.rmtree(
            os.path.expanduser("~/.local/share/fonts-tmp"), ignore_errors=True
        )

    def run(self):
        self._install_tilix()
        self._install_custom_theme()



import os.path
import shutil
from dataclasses import dataclass
from pathlib import Path
from typing import List

from workstation import RESOURCES_PATH
from workstation.os_manager import OSManager
from workstation.zsh import ZSH_CONFIG_PATH

PY_SYS = "/usr/bin/python3"
POETRY_HOME = os.path.expanduser("~/.poetry")
PYENV_ROOT = os.path.expanduser("~/.pyenv")
PYENV_ZSH_CONFIG_LOCATION = os.path.join(ZSH_CONFIG_PATH, "pyenv.zsh")
PYENV_ZSH_CONFIG_RESOURCE_LOCATION = os.path.join(RESOURCES_PATH, "zshrc", "pyenv.zsh")
VIRTUALENV_ZSH_CONFIG_LOCATION = os.path.join(ZSH_CONFIG_PATH, "virtualenv.zsh")
VIRTUALENV_ZSH_CONFIG_RESOURCE_LOCATION = os.path.join(
    RESOURCES_PATH, "zshrc", "virtualenv.zsh"
)
POETRY_ZSH_CONFIG_LOCATION = os.path.join(ZSH_CONFIG_PATH, "poetry.zsh")
POETRY_ZSH_CONFIG_RESOURCE_LOCATION = os.path.join(
    RESOURCES_PATH, "zshrc", "poetry.zsh"
)
POETRY_CMD = f"curl -sSL https://install.python-poetry.org | POETRY_HOME={POETRY_HOME} {PY_SYS} -"


@dataclass
class PythonInstaller:
    os_manager: OSManager

    def global_install(self, *args):
        cmd = f"{PY_SYS} -E -m pip install --upgrade " + " ".join(args) + " --user"
        print("********", cmd)
        self.os_manager.run(cmd)

    @staticmethod
    def _remove_pyenv():
        shutil.rmtree(PYENV_ROOT, ignore_errors=True)
        Path(PYENV_ZSH_CONFIG_LOCATION).unlink(missing_ok=True)

    def _install_pyenv(self):
        self.os_manager.native_install(
            "make",
            "gcc",
            "zlib-devel",
            "bzip2",
            "bzip2-devel",
            "readline-devel",
            "sqlite",
            "sqlite-devel",
            "openssl-devel",
            "tk-devel",
            "libffi-devel",
            "xz-devel",
            sudo=True,
        )
        self._remove_pyenv()
        cmd = "curl -L https://github.com/pyenv/pyenv-installer/raw/master/bin/pyenv-installer | bash"
        self.os_manager.run(cmd)
        shutil.copy(PYENV_ZSH_CONFIG_RESOURCE_LOCATION, PYENV_ZSH_CONFIG_LOCATION)

    def _install_python_versions(self, python_versions: List[str]):
        self.os_manager.run_in_zsh("pyenv update")
        for pv in python_versions:
            cmd = f"pyenv install {pv}"
            self.os_manager.run_in_zsh(cmd)

    def _install_utils(self):
        # TODO: out of the function
        r = ["nox==2021.10.1", "nox-poetry==0.9.0"]
        self.global_install(*r)

    def _install_virtualenv(self):
        r = ["virtualenv==20.10.0", "virtualenvwrapper==4.8.4"]
        self.global_install(*r)
        shutil.copy(
            VIRTUALENV_ZSH_CONFIG_RESOURCE_LOCATION, VIRTUALENV_ZSH_CONFIG_LOCATION
        )

    def _remove_poetry(self):
        Path(POETRY_ZSH_CONFIG_LOCATION).unlink(missing_ok=True)
        cmd = POETRY_CMD + " --uninstall"
        self.os_manager.run(cmd)

    def _install_poetry(self):
        self._remove_poetry()
        cmd = f"curl -sSL https://install.python-poetry.org | POETRY_HOME={POETRY_HOME} {PY_SYS} -"
        self.os_manager.run(cmd)
        shutil.copy(POETRY_ZSH_CONFIG_RESOURCE_LOCATION, POETRY_ZSH_CONFIG_LOCATION)

    def run(self, python_versions: List[str]):
        self._install_pyenv()
        # self._install_python_versions(python_versions=python_versions)
        self._install_virtualenv()
        self._install_poetry()
        self._install_utils()



import importlib.resources as pkg_resources
import os

OS_FEDORA = "fedora"
OS_UBUNTU = "ubuntu"
OS_MAC = "macintosh"

HOME = os.environ["HOME"]
RESOURCES_PATH = str(pkg_resources.path("workstation", "resources"))



import os.path
import shutil
import subprocess
import tarfile
from dataclasses import dataclass
from pathlib import Path

import wget

from workstation import HOME
from workstation import OS_FEDORA


def full_path(suffix: str) -> str:
    return os.path.join(HOME, suffix)


@dataclass
class OSManager:
    operating_system: str

    @staticmethod
    def run_in_zsh(cmd):
        pass

    @staticmethod
    def run(cmd) -> int:
        return os.system(cmd)
        # args = shlex.split(cmd)
        # sub = subprocess.Popen(args, stdout=subprocess.PIPE)
        # return sub

    @staticmethod
    def native_update(**kwargs):
        pass

    def native_install(self, *args, **kwargs):
        pass


@dataclass
class Fedora(OSManager):
    operating_system: str = OS_FEDORA

    @staticmethod
    def run_in_zsh(cmd):
        cmd = "source ~/.zshrc;" + cmd
        subprocess.call(cmd, shell=True, executable="/bin/zsh")

    def native_install(self, *args, **kwargs):
        self.native_update(**kwargs)
        cmd = "dnf install -y "
        if kwargs.get("sudo", False):
            cmd = "sudo " + cmd
        cmd += " ".join(args) + " 2>/dev/null"
        os.system(cmd)

    @staticmethod
    def native_update(**kwargs):
        cmd = "dnf update"
        if kwargs.get("sudo", False):
            cmd = "sudo " + cmd
        os.system(cmd)

    @staticmethod
    def tgz_install_from_url(url: str, binary_name: str):
        dest = os.path.expanduser("~/.local/bin")
        os.makedirs(dest, exist_ok=True)
        file = wget.download(url)
        with tarfile.open(file, "r:gz") as tar:
            tar.extractall()
            folder = tar.getnames()[0]
        shutil.copy(os.path.abspath(os.path.join(folder, binary_name)), dest)
        Path(os.path.abspath(file)).unlink(missing_ok=True)
        shutil.rmtree(folder)

    def rpm_install_from_url(self, url: str):
        file = wget.download(url, "tmp.rpm")
        self.native_install("tpm.rpm", sudo=True)
        Path(os.path.abspath(file)).unlink(missing_ok=True)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['workstation']

package_data = \
{'': ['*'], 'workstation': ['resources/*', 'resources/zshrc/*']}

install_requires = \
['click>=8.0.3,<9.0.0',
 'distro>=1.6.0,<2.0.0',
 'loguru==0.5.3',
 'wget>=3.2,<4.0']

entry_points = \
{'console_scripts': ['install-workstation = '
                     'workstation.cli:install_workstation']}

setup_kwargs = {
    'name': 'workstation-cli',
    'version': '0.1.0',
    'description': 'Python lib for my personal workstation setup',
    'long_description': '# Workstation\n\n![Tests](https://github.com/militu/workstation/actions/workflows/tests.yml/badge.svg)\n![Docs](https://github.com/militu/workstation/actions/workflows/documentation.yml/badge.svg)\n\n### 🕮 Documentation\n\nFor detailed documentation (todo), please see [here](https://militu.github.io/workstation/)\n\n```shell\npip install workstation\n```\n\n### \U0001fa9b Develop\n\nFirst install workstation\n\n```shell\npyenv install -s 3.8.12\npyenv local 3.8.12\npoetry lock\nnox -s pre-commit -- install\nnox -s pre-commit -- install --hook-type commit-msg\n```\n',
    'author': 'Victor Mazzeo',
    'author_email': 'victor.mazzeo@groupeseloger.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/militu/workstation-cli',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import os.path
import shutil
from dataclasses import dataclass
from pathlib import Path

from workstation import RESOURCES_PATH
from workstation.os_manager import OSManager
from workstation.zsh import ZSH_CONFIG_PATH

NVM_ROOT = os.path.expanduser("~/.nvm")
NVM_ZSH_CONFIG_LOCATION = os.path.join(ZSH_CONFIG_PATH, "nvm.zsh")
NVM_ZSH_CONFIG_RESOURCE_LOCATION = os.path.join(RESOURCES_PATH, "zshrc", "nvm.zsh")


@dataclass
class NVMInstaller:
    os_manager: OSManager

    @staticmethod
    def _remove_nvm():
        shutil.rmtree(NVM_ROOT, ignore_errors=True)

    def _install_nvm(self):
        self._remove_nvm()
        cmd = f"git clone https://github.com/nvm-sh/nvm.git {NVM_ROOT}"
        self.os_manager.run(cmd)

    def _install_node(self, node_version: str):
        cmd = f"nvm install {node_version}"
        self.os_manager.run_in_zsh(cmd)

    @staticmethod
    def _remove_nvm_zsh_config():
        Path(NVM_ZSH_CONFIG_LOCATION).unlink(missing_ok=True)

    def _setup_nvm_zsh_config(self):
        self._remove_nvm_zsh_config()
        shutil.copy(NVM_ZSH_CONFIG_RESOURCE_LOCATION, NVM_ZSH_CONFIG_LOCATION)

    def run(self, node_version: str):
        self._install_nvm()
        self._setup_nvm_zsh_config()
        self._install_node(node_version=node_version)
